/*
 * mpeg2cleaner - Entfernt 'zero byte stuffing'-Bereiche aus MPEG2-ES-Dateien.
 * Homepage: http://basicmaster.de/mpeg2cleaner/
 * 
 * Copyright 2011-2015 Stefan Pöschel
 * Das Programm darf kostenlos zu privaten, nichtkommerziellen Zwecken benutzt werden (Benutzung auf eigene Gefahr).
 */


#include <climits>
#include <clocale>
#include <cmath>
#include <cstdlib>
#include <ctime>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <stdint.h>

using namespace std;

#define SOURCE_BUFFER_SIZE		(1024*1024)
#define SLICE_MAX				0xAF
#define MPEG2CLEANER_VERSION	"v1.4"
#define MPEG2CLEANER_YEAR		"2011-15"

// german format
struct german_numpunct : numpunct<char> {
	char do_decimal_point() const {return ',';}
	string do_grouping() const {return "\3";}
	char do_thousands_sep() const {return '.';}
};


// --- sStatistics -----------------------------------------------------------------
struct sStatistics {
	int count;
	int fillbytes_total;
	int fillbytes_min;
	int fillbytes_max;

	sStatistics() : count(0), fillbytes_total(0), fillbytes_min(INT_MAX), fillbytes_max(0) {}
	void ApplyRegion(int removed_bytes_region);
};


// --- cMPEG2Cleaner -----------------------------------------------------------------
class cMPEG2Cleaner {
private:
	const char *source_filename;
	const char *target_filename;
	bool mode_verbose;
	bool mode_write;

	ifstream source_file;
	ofstream target_file;
	uint64_t source_len;

	uint8_t *source_buffer;
	uint64_t source_buffer_level;

	uint8_t curr_slice;
	unsigned int rb_offset;
	int data_offset;
	int fillbytes_region;
	int fillbytes_pic;

	int progress;
	double filler_percentage;
	clock_t timestamp_start;

	sStatistics slice_stat[SLICE_MAX + 1];
	sStatistics pic_stat;

	void FillSourceBuffer();
	void UpdateProgress(const bool initial_call);
	double CurrentDurationSec() {return (double) (clock() - timestamp_start) / CLOCKS_PER_SEC;}

	void FillerStart() {fillbytes_region = 0;}
	void FillerEnd();

	void PicBoundary();

	void DataStart() {data_offset = rb_offset;}
	void DataBoundary(const bool data_continues);
public:
	cMPEG2Cleaner(const char *source_filename, const char *target_filename, const bool mode_verbose, const bool mode_write);
	~cMPEG2Cleaner();
	void RemoveZeroByteStuffing();
};
